﻿using System.Linq;
using System.Threading.Tasks;
using Microsoft.Extensions.Options;
using SiteStatus.Dbo;

namespace SiteStatus.Business
{
    public class VpnBusiness
    {
        public HostBusiness HostBusiness { get; }
        public SettingsDbo Options { get; }

        public VpnBusiness(HostBusiness hostBusiness, IOptions<SettingsDbo> options)
        {
            HostBusiness = hostBusiness;
            Options = options.Value;
        }

        public VpnStatusDbo GetVpnStatus()
        {
            var sites = Options.VpnS2s.Sites.Select(site => new SiteStatusDbo
            {
                Name = string.IsNullOrEmpty(site.Name) ? null : site.Name,
                Hosts = site.Hosts.Select(host => new HostStatusDbo
                {
                    Ip = host.Ip
                }).ToList()
            }).ToList();

            var hosts = sites.SelectMany(site => site.Hosts).ToList();
            var tasks = hosts.Select(host => HostBusiness.GetHostStatusAsync(host.Ip, Options.VpnS2s.Ping.Count, Options.VpnS2s.Ping.Timeout)).ToArray();

            Task.WaitAll(tasks);

            foreach (var task in tasks)
            {
                foreach (var site in sites)
                {
                    var hostStatus = site.Hosts.FirstOrDefault(host => host.Ip == task.Result.Ip.ToString());
                    if (hostStatus != null)
                    {
                        var idx = site.Hosts.IndexOf(hostStatus);
                        site.Hosts[idx] = task.Result;
                    }
                }
            }

            return new VpnStatusDbo
            {
                Sites = sites
            };
        }
    }
}